@@ -1380,7 +1380,8 @@ bool pkgProblemResolver::ResolveByKeepInternal()
 	       pkgCache::PkgIterator Pkg = Ver.ParentPkg();
 	       
 	       // It is not keepable
-	       if (Pkg->CurrentVer == 0)
+	       if (Cache[Pkg].InstallVer == 0 ||
+		   Pkg->CurrentVer == 0)
 		  continue;
 
 	       if (not Cache[Pkg].Protect())
@@ -1423,9 +1424,6 @@ bool pkgProblemResolver::ResolveByKeepInternal()
 
    delete[] PList;
 
-   if (Cache.BrokenCount() != 0)
-      return _error->Error(_("Unable to correct problems, you have held broken packages."));
-
    if (Debug)
       Cache.CheckConsistency("keep done");
 
