@@ -25,11 +25,11 @@
 #include <apt-pkg/tagfile.h>
 
 #include <algorithm>
+#include <cctype>
+#include <cstddef>
+#include <cstring>
 #include <string>
 #include <vector>
-#include <ctype.h>
-#include <stddef.h>
-#include <string.h>
 									/*}}}*/
 
 using std::string;
@@ -282,25 +282,7 @@ std::vector<std::string> debListParser::AvailableDescriptionLanguages()
  */
 APT::StringView debListParser::Description_md5()
 {
-   StringView const value = Section.Find(pkgTagSection::Key::Description_md5);
-   if (unlikely(value.empty() == true))
-   {
-      StringView const desc = Section.Find(pkgTagSection::Key::Description);
-      if (desc == "\n")
-	 return StringView();
-
-      Hashes md5(Hashes::MD5SUM);
-      md5.Add(desc.data(), desc.size());
-      md5.Add("\n");
-      MD5Buffer = md5.GetHashString(Hashes::MD5SUM).HashValue();
-      return StringView(MD5Buffer);
-   }
-   else if (likely(value.size() == 32))
-   {
-      return value;
-   }
-   _error->Error("Malformed Description-md5 line; doesn't have the required length (32 != %d) '%.*s'", (int)value.size(), (int)value.length(), value.data());
-   return StringView();
+   return Section.Find(pkgTagSection::Key::Description_md5);
 }
                                                                         /*}}}*/
 // ListParser::UsePackage - Update a package structure			/*{{{*/
@@ -540,7 +522,17 @@ const char *debListParser::ConvertRelation(const char *I,unsigned int &Op)
       Op = pkgCache::Dep::Equals;
       I++;
       break;
-      
+
+      // != is unsupported packaging
+      case '!':
+      if (*(I + 1) == '=')
+      {
+	 I = I + 2;
+	 Op = pkgCache::Dep::NotEquals;
+	 break;
+      }
+      [[fallthrough]];
+
       // HACK around bad package definitions
       default:
       Op = pkgCache::Dep::Equals;
@@ -616,7 +608,7 @@ const char *debListParser::ParseDepends(const char *Start, const char *Stop,
    {
       // Skip the '('
       for (I++; I != Stop && isspace_ascii(*I) != 0 ; I++);
-      if (I + 3 >= Stop)
+      if (I + 3 > Stop)
 	 return 0;
       I = ConvertRelation(I,Op);
       
@@ -643,63 +635,60 @@ const char *debListParser::ParseDepends(const char *Start, const char *Stop,
    // Skip whitespace
    for (;I != Stop && isspace_ascii(*I) != 0; I++);
 
-   if (unlikely(ParseArchFlags == true))
+   // Parse architecture restrictions
+   if (ParseArchFlags && I != Stop && *I == '[')
    {
+      for (++I; I != Stop && isspace_ascii(*I) != 0 && *I != ']'; ++I);
+      // malformed
+      if (unlikely(I == Stop || *I == ']'))
+	 return 0;
+
       APT::CacheFilter::PackageArchitectureMatchesSpecification matchesArch(Arch, false);
 
-      // Parse an architecture
-      if (I != Stop && *I == '[')
+      bool Found = false;
+      bool NegArch = false;
+      while (I < Stop && *I != ']')
       {
-	 ++I;
-	 // malformed
-	 if (unlikely(I == Stop))
+	 // look for whitespace or ending ']' to end current Arch
+	 const char *End = I;
+	 for (;End < Stop && isspace_ascii(*End) == 0 && *End != ']'; ++End);
+
+	 if (unlikely(End >= Stop))
 	    return 0;
 
-	 const char *End = I;
-	 bool Found = false;
-	 bool NegArch = false;
-	 while (I != Stop)
+	 bool CurNegArch = false;
+	 if (*I == '!')
 	 {
-	    // look for whitespace or ending ']'
-	    for (;End != Stop && !isspace_ascii(*End) && *End != ']'; ++End);
-
-	    if (unlikely(End == Stop))
-	       return 0;
-
-	    if (*I == '!')
-	    {
-	       NegArch = true;
-	       ++I;
-	    }
-
-	    std::string const arch(I, End);
-	    if (arch.empty() == false && matchesArch(arch.c_str()) == true)
-	    {
-	       Found = true;
-	       if (I[-1] != '!')
-		  NegArch = false;
-	       // we found a match, so fast-forward to the end of the wildcards
-	       for (; End != Stop && *End != ']'; ++End);
-	    }
-
-	    if (*End++ == ']') {
-	       I = End;
-	       break;
-	    }
-
-	    I = End;
-	    for (;I != Stop && isspace_ascii(*I) != 0; I++);
+	    NegArch = true;
+	    CurNegArch = true;
+	    ++I;
 	 }
 
-	 if (NegArch == true)
-	    Found = !Found;
+	 if (I >= End)
+	    return 0;
+	 std::string const arch(I, End);
+	 if (matchesArch(arch.c_str()))
+	 {
+	    Found = true;
+	    if (not CurNegArch)
+	       NegArch = false;
+	    // we found a match, so fast-forward to the end of the wildcards
+	    for (; End < Stop && *End != ']'; ++End);
+	 }
+	 else
+	    for (; End < Stop && isspace_ascii(*End) != 0; ++End);
 
-	 if (Found == false)
-	    Package = ""; /* not for this arch */
+	 I = End;
       }
 
+      if (NegArch == true)
+	 Found = not Found;
+
+      if (Found == false)
+	 Package = ""; /* not for this arch */
+
       // Skip whitespace
-      for (;I != Stop && isspace_ascii(*I) != 0; I++);
+      for (++I; I < Stop && isspace_ascii(*I) != 0; ++I);
    }
 
    if (unlikely(ParseRestrictionsList == true))
@@ -895,7 +884,7 @@ bool debListParser::ParseProvides(pkgCache::VerIterator &Ver)
    bool const barbarianArch = not APT::Configuration::checkArchitecture(Arch);
    const char *Start;
    const char *Stop;
-   if (Section.Find(pkgTagSection::Key::Provides,Start,Stop) == true)
+   if (Section.Find(pkgTagSection::Key::Provides,Start,Stop) && Start != Stop)
    {
       StringView Package;
       StringView Version;
