@@ -24,15 +24,16 @@
 #include <apt-pkg/strutl.h>
 #include <apt-pkg/version.h>
 
+#include <cerrno>
+#include <csignal>
+#include <cstddef>
+#include <cstdio>
+#include <cstdlib>
+#include <ctime>
 #include <dirent.h>
-#include <errno.h>
 #include <fcntl.h>
 #include <grp.h>
 #include <pwd.h>
-#include <signal.h>
-#include <stddef.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <sys/ioctl.h>
 #include <sys/select.h>
 #include <sys/stat.h>
@@ -40,7 +41,6 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <termios.h>
-#include <time.h>
 #include <unistd.h>
 
 #include <algorithm>
@@ -1301,7 +1301,7 @@ void pkgDPkgPM::StartPtyMagic()						/*{{{*/
 	 free(d->slave);
 	 d->slave = NULL;
       }
-      _error->DumpErrors(std::cerr, GlobalError::DEBUG, false);
+      _error->DumpErrors(std::cerr, GlobalError::NOTICE, false);
    }
    _error->RevertToStack();
 }
