@@ -42,6 +42,7 @@
 #include <unordered_map>
 #include <utility>
 #include <vector>
+#include <dirent.h>
 
 #include <sys/stat.h>
 
@@ -1462,7 +1463,7 @@ static bool MarkInstall_RemoveConflictsIfNotUpgradeable(pkgDepCache &Cache, bool
    return not failedToRemoveSomething;
 }
 									/*}}}*/
-static bool MarkInstall_CollectReverseDepends(pkgDepCache &Cache, bool const DebugAutoInstall, pkgCache::VerIterator const &PV, unsigned long Depth, APT::PackageVector &toUpgrade) /*{{{*/
+static bool MarkInstall_CollectReverseDepends(pkgDepCache &Cache, bool const DebugAutoInstall, pkgCache::VerIterator const &PV, unsigned long Depth, APT::PackageVector &toUpgrade, APT::PackageVector const &delayedRemove) /*{{{*/
 {
    auto CurrentVer = PV.ParentPkg().CurrentVer();
    if (CurrentVer.end())
@@ -1473,6 +1474,9 @@ static bool MarkInstall_CollectReverseDepends(pkgDepCache &Cache, bool const Deb
       // Skip non-installed versions and packages already marked for upgrade
       if (ParentPkg.CurrentVer() != D.ParentVer() || Cache[ParentPkg].Install())
 	 continue;
+      // Skip rev-depends we already tagged for removal
+      if (Cache[ParentPkg].Delete() || std::find(delayedRemove.begin(), delayedRemove.end(), ParentPkg) != delayedRemove.end())
+	 continue;
       // We only handle important positive dependencies, RemoveConflictsIfNotUpgradeable handles negative
       if (not Cache.IsImportantDep(D) || D.IsNegative())
 	 continue;
@@ -1721,7 +1725,7 @@ bool pkgDepCache::MarkInstall(PkgIterator const &Pkg, bool AutoInst,
 	    return false;
 	 hasFailed = true;
       }
-      if (not MarkInstall_CollectReverseDepends(*this, DebugAutoInstall, PV, Depth, toUpgrade))
+      if (not MarkInstall_CollectReverseDepends(*this, DebugAutoInstall, PV, Depth, toUpgrade, delayedRemove))
       {
 	 if (failEarly)
 	    return false;
@@ -2451,6 +2455,25 @@ static bool MarkPackage(pkgCache::PkgIterator const &Pkg,
 
       if (not unsatisfied_choice)
 	 fullyExplored[T->ID] = true;
+
+      // do not follow newly installed providers if we have already installed providers
+      if (providers_by_source.size() >= 2)
+      {
+	 if (std::any_of(providers_by_source.begin(), providers_by_source.end(), [](auto const PV) {
+			 return std::any_of(PV.second.begin(), PV.second.end(), [](auto const &Prv) {
+			   auto const PP = Prv.ParentPkg();
+			   return not PP.end() && PP->CurrentVer != 0;
+			 });}))
+	 {
+	    for (auto &providers : providers_by_source)
+	       providers.second.erase(std::remove_if(providers.second.begin(), providers.second.end(),
+			[](auto const &Prv) {
+			   auto const PP = Prv.ParentPkg();
+			   return not PP.end() && PP->CurrentVer == 0;
+			}), providers.second.end());
+	 }
+      }
+
       for (auto const &providers : providers_by_source)
       {
 	 for (auto const &PV : providers.second)
@@ -2613,3 +2636,58 @@ bool pkgDepCache::PhasingApplied(pkgCache::PkgIterator Pkg) const
    return true;
 }
 									/*}}}*/
+
+// DepCache::BootSize						/*{{{*/
+double pkgDepCache::BootSize(bool initrdOnly)
+{
+   double BootSize = 0;
+   int BootCount = 0;
+   auto VirtualKernelPkg = FindPkg("$kernel", "any");
+   if (VirtualKernelPkg.end())
+      return 0;
+
+   for (pkgCache::PrvIterator Prv = VirtualKernelPkg.ProvidesList(); Prv.end() == false; ++Prv)
+   {
+      auto Pkg = Prv.OwnerPkg();
+      if ((*this)[Pkg].NewInstall())
+	 BootSize += (*this)[Pkg].InstallVer->InstalledSize, BootCount++;
+   }
+   if (BootCount == 0)
+      return 0;
+   if (initrdOnly)
+      BootSize = 0;
+
+   DIR *boot = opendir(_config->FindDir("Dir::Boot").c_str());
+   struct dirent *ent;
+   if (boot)
+   {
+      double initrdSize = 0;
+      double mapSize = 0;
+      while ((ent = readdir(boot)))
+      {
+	 enum
+	 {
+	    INITRD,
+	    MAP
+	 } type;
+	 if (APT::String::Startswith(ent->d_name, "initrd.img-"))
+	    type = INITRD;
+	 else if (APT::String::Startswith(ent->d_name, "System.map-"))
+	    type = MAP;
+	 else
+	    continue;
+
+	 auto path = _config->FindDir("Dir::Boot") + ent->d_name;
+
+	 if (struct stat st; stat(path.c_str(), &st) == 0)
+	 {
+	    double &targetSize = type == INITRD ? initrdSize : mapSize;
+	    targetSize = std::max(targetSize, double(st.st_size));
+	 }
+      }
+      closedir(boot);
+      return initrdSize ? BootSize + BootCount * (initrdSize + mapSize) * 1.1 : 0;
+   }
+   return 0;
+}
+									/*}}}*/
