@@ -202,10 +202,10 @@ void APT::Solver::Work::Dump(pkgCache &cache)
    if (optional)
       std::cerr << "Optional ";
    std::cerr << "Item (" << ssize_t(size <= solutions.size() ? size : -1) << "@" << depth << (upgrade ? "u" : "") << ") ";
-   if (auto Pkg = reason.Pkg(); Pkg != 0)
-      std::cerr << pkgCache::PkgIterator(cache, cache.PkgP + Pkg).FullName();
-   if (auto Ver = reason.Ver(); Ver != 0)
-      std::cerr << pkgCache::VerIterator(cache, cache.VerP + Ver).ParentPkg().FullName() << "=" << pkgCache::VerIterator(cache, cache.VerP + Ver).VerStr();
+   if (auto Pkg = reason.Pkg(cache); not Pkg.end())
+      std::cerr << Pkg.FullName();
+   if (auto Ver = reason.Ver(cache); not Ver.end())
+      std::cerr << Ver.ParentPkg().FullName() << "=" << Ver.VerStr();
    std::cerr << " -> ";
    for (auto sol : solutions)
    {
@@ -221,7 +221,7 @@ std::string APT::Solver::WhyStr(Reason reason)
 
    while (not reason.empty())
    {
-      if (auto Pkg = pkgCache::PkgIterator(cache, cache.PkgP + reason.Pkg()); not Pkg.end())
+      if (auto Pkg = reason.Pkg(cache); not Pkg.end())
       {
 	 if ((*this)[Pkg].decision == Decision::MUSTNOT)
 	    out.push_back(std::string("not ") + Pkg.FullName());
@@ -229,7 +229,7 @@ std::string APT::Solver::WhyStr(Reason reason)
 	    out.push_back(Pkg.FullName());
 	 reason = (*this)[Pkg].reason;
       }
-      if (auto Ver = pkgCache::VerIterator(cache, cache.VerP + reason.Ver()); not Ver.end())
+      if (auto Ver = reason.Ver(cache); not Ver.end())
       {
 	 if ((*this)[Ver].decision == Decision::MUSTNOT)
 	    out.push_back(std::string("not ") + Ver.ParentPkg().FullName() + "=" + Ver.VerStr());
@@ -247,8 +247,39 @@ std::string APT::Solver::WhyStr(Reason reason)
    return outstr;
 }
 
-bool APT::Solver::Obsolete(pkgCache::PkgIterator pkg)
+// This is essentially asking whether any other binary in the source package has a higher candidate
+// version. This pretends that each package is installed at the same source version as the package
+// under consideration.
+bool APT::Solver::ObsoletedByNewerSourceVersion(pkgCache::VerIterator cand) const
 {
+   const auto pkg = cand.ParentPkg();
+   const int candPriority = policy.GetPriority(cand);
+
+   for (auto ver = cand.Cache()->FindGrp(cand.SourcePkgName()).VersionsInSource(); not ver.end(); ver = ver.NextInSource())
+   {
+      // We are only interested in other packages in the same source package; built for the same architecture.
+      if (ver->ParentPkg == cand->ParentPkg || ver.ParentPkg()->Arch != cand.ParentPkg()->Arch || cache.VS->CmpVersion(ver.SourceVerStr(), cand.SourceVerStr()) <= 0)
+	 continue;
+
+      // We also take equal priority here, given that we have a higher version
+      const int priority = policy.GetPriority(ver, true);
+      if (priority == 0 || priority < candPriority)
+	 continue;
+
+      pkgObsolete[pkg->ID] = 2;
+      if (debug >= 3)
+	 std::cerr << "Obsolete: " << cand.ParentPkg().FullName() << "=" << cand.VerStr() << " due to " << ver.ParentPkg().FullName() << "=" << ver.VerStr() << "\n";
+      return true;
+   }
+
+   return false;
+}
+
+bool APT::Solver::Obsolete(pkgCache::PkgIterator pkg) const
+{
+   if (pkgObsolete[pkg->ID] != 0)
+      return pkgObsolete[pkg->ID] == 2;
+
    auto ver = policy.GetCandidateVer(pkg);
 
    if (ver.end() && not StrictPinning)
@@ -256,18 +287,13 @@ bool APT::Solver::Obsolete(pkgCache::PkgIterator pkg)
    if (ver.end())
    {
       std::cerr << "Obsolete: " << pkg.FullName() << " - not installable\n";
+      pkgObsolete[pkg->ID] = 2;
       return true;
    }
-   if (pkgObsolete[pkg->ID] != 0)
-      return pkgObsolete[pkg->ID] == 2;
-   for (auto bin = ver.Cache()->FindGrp(ver.SourcePkgName()).VersionsInSource(); not bin.end(); bin = bin.NextInSource())
-      if (bin != ver && bin.ParentPkg()->Arch == ver.ParentPkg()->Arch && bin->ParentPkg != ver->ParentPkg && (not StrictPinning || policy.GetCandidateVer(bin.ParentPkg()) == bin) && _system->VS->CmpVersion(bin.SourceVerStr(), ver.SourceVerStr()) > 0)
-      {
-	 pkgObsolete[pkg->ID] = 2;
-	 if (debug >= 3)
-	    std::cerr << "Obsolete: " << ver.ParentPkg().FullName() << "=" << ver.VerStr() << " due to " << bin.ParentPkg().FullName() << "=" << bin.VerStr() << "\n";
-	 return true;
-      }
+
+   if (ObsoletedByNewerSourceVersion(ver))
+      return true;
+
    for (auto file = ver.FileList(); !file.end(); file++)
       if ((file.File()->Flags & pkgCache::Flag::NotSource) == 0)
       {
