@@ -51,7 +51,11 @@ bool DoUpdate(CommandLine &CmdL)
 {
    if (CmdL.FileSize() != 1)
       return _error->Error(_("The update command takes no arguments"));
+   return DoUpdate();
+}
 
+bool DoUpdate()
+{
    CacheFile Cache;
 
    // Get the source list
@@ -229,6 +233,20 @@ bool DoUpdate(CommandLine &CmdL)
       }
    }
 
+   if (_config->FindB("APT::Get::Update::SourceListWarnings::SignedBy", SLWarnings))
+   {
+      for (auto *S : *List)
+      {
+	 if (not S->HasFlag(metaIndex::Flag::DEB822) || not S->GetSignedBy().empty())
+	    continue;
+
+	 URI uri(S->GetURI());
+	 // TRANSLATOR: the first is manpage reference, the last the URI from a sources.list
+	 _error->Notice(_("Missing Signed-By in the %s entry for '%s'"),
+			"sources.list(5)", URI::ArchiveOnly(uri).c_str());
+      }
+   }
+
    // show basic stats (if the user whishes)
    if (_config->FindB("APT::Cmd::Show-Update-Stats", false) == true)
    {
@@ -248,7 +266,11 @@ bool DoUpdate(CommandLine &CmdL)
       if (upgradable == 0)
          c1out << _("All packages are up to date.") << std::endl;
       else
-         ioprintf(c1out, msg, upgradable);
+      {
+	 c1out << _config->Find("APT::Color::Bold");
+	 ioprintf(c1out, msg, upgradable);
+	 c1out << _config->Find("APT::Color::Neutral");
+      }
 
       RunScripts("APT::Update::Post-Invoke-Stats");
    }
